package de.fhdo.gerding.wifibuttonled;

import java.lang.reflect.Array;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketTimeoutException;
import java.util.Arrays;

/**
* Klasse, die eine Kommunikation mit einem Arduino + RedFly/WiFly Shield ermglicht. Sie beinhaltet Methoden zum senden und empfangen
* digitaler und analoger Werte und zustzlich das Senden eines PWM Signals fr die Servo-Steuerung
*
*/
public class ArduinoCommManager 
{
		
	public ArduinoCommManager() 
	{
		super();
	}
	
	/**
	* Sendet dem Arduino HIGH oder LOW fr den jeweiligen Pin. LOW bedeutet aus (0 Volt). HIGH bedeutet an (5 Volt).
	*
	* @param pin der spezifische Pin, an dem das Signal gesendet werden soll (4 bis 19 (14-19 = A0-A5))
	* @param state der zu bertragende Zustand (true=HIGH, false=LOW)
	* @param address die Adresse des Arduino (bzw. des RedFly/WiFly - Shields)
	* @param port Port des Arduino
	*/
	public void sendDigitalCommand(int pin, boolean state, InetAddress address, int port) throws Exception
	{
		if (pin >= 4 && pin <= 19) 
		{
			char stateChar;
			
			if(state)
				stateChar = (char)1;
			else
				stateChar = (char)0;
			
			String commString = "D" + (char)pin + stateChar;
	        DatagramSocket socket = new DatagramSocket(); 
	        byte[] buf = commString.getBytes();  
	        DatagramPacket packet = new DatagramPacket(buf, buf.length, address, port);  
	        socket.send(packet);
	        socket.close();
		}else{
			throw new Exception("Ungueltiger Pin!");
		}
	}
	
	/**
	* Sendet dem Arduino einen analogen Befehl fr den jeweiligen Pin. Dieser verndert die anliegende Spannung
	* an dem spezifischen Pin je nach Auslenkung (0 bis 100 Prozent [max. 5 Volt]).
	*
	* @param pin der spezifische Pin, an dem das Signal gesendet werden soll (4 bis 13)
	* @param state der zu bertragende Zustand (0 bis 100)
	* @param address die Adresse des Arduino (bzw. des RedFly/WiFly - Shields)
	* @param port Port des Arduino
	*/
	public void sendAnalogCommand(int pin, int state, InetAddress address, int port) throws Exception{
		if (pin >= 4 && pin <= 13) {
			if(state >= 0 && state <= 100){
				String commString = "A" + (char)pin + (char)state;
				
	            DatagramSocket socket = new DatagramSocket(); 
	            byte[] buf = commString.getBytes("UTF-8");  
	            DatagramPacket packet = new DatagramPacket(buf, buf.length, address, port);  
	            socket.send(packet);
	            socket.close();
			}else{
				throw new Exception("Ungueltiger Zustands-Wert (muss zwischen 0 und 100 liegen)");
			}
		}else{
			throw new Exception("Ungueltiger Pin!");
		}
	}
	
	/**
	* Sendet dem Arduino ein PWM Signal fr den jeweiligen Pin. Dieser Befehl ist fr Servos gedacht. Je nach
	* Auslenkung (0 bis 100 Prozent) bewegt sich der Servo in die eine oder entgegengesetzte Richtung.
	*
	* @param pin der spezifische Pin, an dem das Signal gesendet werden soll (4 bis 13)
	* @param state der zu bertragende Zustand (0 bis 100)
	* @param address die Adresse des Arduino (bzw. des RedFly/WiFly - Shields)
	* @param port Port des Arduino
	*/
	public void sendServoCommand(int pin, int state, InetAddress address, int port) throws Exception{
		if (pin >= 4 && pin <= 13) {
			if(state >= 0 && state <= 100){
				String commString = "S" + (char)pin + (char)state;
				
	            DatagramSocket socket = new DatagramSocket(); 
	            byte[] buf = commString.getBytes("UTF-8");  
	            DatagramPacket packet = new DatagramPacket(buf, buf.length, address, port);  
	            socket.send(packet);
	            socket.close();
			}else{
				throw new Exception("Ungueltiger Zustands-Wert (muss zwischen 0 und 100 liegen)");
			}
		}else{
			throw new Exception("Ungueltiger Pin!");
		}
	}
	
	/**
	* Liest den analogen Wert an dem spezifischen Pin des Arduino.
	*
	* @param pin der spezifische Pin, an dem der Zustand gelesen werden soll (14 bis 19 (14-19 = A0-A5))
	* @param address die Adresse des Arduino (bzw. des RedFly/WiFly - Shields)
	* @param port Port des Arduino
	* @return den an dem spezifischen Pin anliegenden Zustand (0 bis 100)
	*/
	public int readAnalogValue(int pin, InetAddress address, int port) throws Exception{
		if (pin >= 14 && pin <= 19) {
			String commString = "B" + (char)pin + (char)0;
				
	        DatagramSocket socket = new DatagramSocket(1234); 
	        socket.setSoTimeout(4000);
	        byte[] buf = commString.getBytes("UTF-8");  
	        DatagramPacket packet = new DatagramPacket(buf, buf.length, address, port);  
	        socket.send(packet);
	        
	        byte[] buf_receive = new byte[1];
	        DatagramPacket packet_receive = new DatagramPacket(buf_receive, buf_receive.length);
	        do {
	        	try{
	        		socket.receive(packet_receive);
	        	}catch(SocketTimeoutException ex){
	        		socket.close();
	        		throw new Exception("Arduino-Board antwortet nicht! Timeout!!");
	        	}
	        } while (!packet_receive.getAddress().equals(address) || packet_receive.getPort() != port);

	        byte[] data = packet_receive.getData();
	        int value = Array.getInt(data, 0);
	        socket.close();
	        return value;
		}else{
			throw new Exception("Ungueltiger Pin!");
		}
	}
	
	/**
	* Liest den digitalen Wert an dem spezifischen Pin des Arduino.
	*
	* @param pin der spezifische Pin, an dem der Zustand gelesen werden soll (4 bis 19 (14-19 = A0-A5))
	* @param address die Adresse des Arduino (bzw. des RedFly/WiFly - Shields)
	* @param port Port des Arduino
	* @return den an dem spezifischen Pin anliegenden Zustand (false=LOW, true=HIGH)
	*/
	public boolean readDigitalValue(int pin, InetAddress address, int port) throws Exception{
		if (pin >= 4 && pin <= 19) {
			String commString = "E" + (char)pin + (char)0;
			
	        DatagramSocket socket = new DatagramSocket(1234);
	        socket.setSoTimeout(4000);
	        byte[] buf = commString.getBytes("UTF-8");  
	        DatagramPacket packet = new DatagramPacket(buf, buf.length, address, port);  
	        socket.send(packet);
	        
	        byte[] buf_receive = new byte[1];
	        DatagramPacket packet_receive = new DatagramPacket(buf_receive, buf_receive.length);
	        do {
	        	try{
	        		socket.receive(packet_receive);
	        	}catch(SocketTimeoutException ex){
	        		socket.close();
	        		throw new Exception("Arduino-Board antwortet nicht! Timeout!!");
	        	}
	        } while (!packet_receive.getAddress().equals(address) || packet_receive.getPort() != port);
	        
	        byte[] data = packet_receive.getData();
	        socket.close();
	        
	        if(Arrays.equals(data,new byte[]{0})){
	        	return false;
	        }else if(Arrays.equals(data,new byte[]{1})){
	        	return true;
	        }else{
	        	throw new Exception("Ungueltige Antwort erhalten!");
	        }
		}else{
			throw new Exception("Ungueltiger Pin!");
		}
	}

}
